#!/bin/bash

#####################################################################
########### ft-G15-Amarok v1.1  (released under GPL 2.0) ############
#####################################################################
# Author:                        Jocke "firetech" Andersson         #
#        (Based on 'G15 Amarok Plugin Plus v1.0' by Robert Riemann) #
# Report bugs and requests to:   jocke@firetech.nu                  #
# Requires:                      g15composer, iconv, amarok, dcop   #
#####################################################################
# The script shows the meta info to the actual track of amarok      #
# meta infos on screen: title, artist, album and length             #
#####################################################################

# Check installation of needed apps
function checkBinary {
	if [ -z $(which $1) ]; then
		echo "${1} could not be found in your PATH. Aborting." >&2
		exit 1
	fi
}
checkBinary g15composer
checkBinary iconv
checkBinary dcop

# Make sure amarok is running and that we can dcop to it
dcop amarok player status > /dev/null 2>&1
if [ "$?" != "0" ]; then
	echo "Could not dcop to Amarok (Is it running?) Aborting." >&2
	exit 1
fi

# Set fifo pipe to use
pipe='/tmp/ft-g15-amarok'

# Check for an already running g15composer with our pipe - Normally there is'nt one
function getComposer {
	pgrep -f -x "g15composer ${pipe}"
}
function checkComposer {
	local pid=$(getComposer)
	if [ -z "${pid}" ]; then
		[ -e "${pipe}" -a ! -p "${pipe}" ] && rm -f $pipe
		g15composer $pipe &
		while [ ! -p "${pipe}" ]; do
			sleep 1
		done
	fi
}

# Get control about the signals to stop the g15composer and remove the pipe
function exit_cleanly {
	local pid=$(getComposer)
	[ -n "${pid}" ] && kill $pid
	[ -e "${pipe}" ] && rm -f $pipe
	exit 0
}
trap "exit_cleanly" HUP INT QUIT FPE KILL TERM

# Construct three dots at specified coordinates (X Y)
function make_dots {
	cat << EOF

PS ${1} $(($2+6)) 1
PS $(($1+3)) $(($2+6)) 1
PS $(($1+6)) $(($2+6)) 1
EOF
}

# Send the Screen layout to G15composer
function send_screen {
	checkComposer

	iconv -t IBM437 -c > $pipe
}

# Main loop
if [ $(dcop amarok player status) -gt 0 ]; then
	amarok_stdin="trackChange"
else
	amarok_stdin="engineStateChange: empty"
fi
while true; do
	# Show the meta infos if amarok is'nt stopped
	if [ "${amarok_stdin}" = "trackChange" -o "${amarok_stdin}" = "engineStateChange: paused" \
		-o "${amarok_stdin}" = "engineStateChange: playing" ]; then
		first_run=0
		# Prepares the title, artist and album
		artistdots=""
		titledots=""
		albumdots=""
		artist=$(dcop amarok player artist)
		if [ ${#artist} -gt 19 ]; then
			artist="${artist:0:18} "
			artistdots=$(make_dots 149 2)
		fi
		title=$(dcop amarok player title)
		if [ ${#title} -gt 19 ]; then
			title="${title:0:18} "
			titledots=$(make_dots 149 10)
		fi
		album="($(dcop amarok player album))"
		if [ ${#album} -gt 19 ]; then
			album="${album:0:17} )"
			albumdots=$(make_dots 141 22)
		fi

		if [ $(dcop amarok player status) = 1 ]; then
			paused=" (paused)"
		else
			paused=""
		fi

		send_screen << EOF # Reads the following lines up to EOF and sends it to our pipe
MC 1
TO 2 2 2 1 "${artist}"${artistdots}
TO 2 10 2 1 "${title}"${titledots}
TO 2 22 2 1 "${album}"${albumdots}
DL 1 32 158 32 1
TO 2 34 1 1 "$(dcop amarok player totalTime)${paused}"
DR 0 0 159 42 1
MC 0
EOF

	# Shows the amarok stopped status screen
	elif [ "${amarok_stdin}" = "engineStateChange: empty" ]; then
		send_screen << EOF # Reads the following lines up to EOF and sends it to our pipe
MC 1
TO 1 18 2 1 "No track playing"
DR 0 0 159 42 1
MC 0
EOF
	fi

	read amarok_stdin
done
